<?php 
/*
Plugin Name: Hackadelic EditArea
Version: 1.1.1
Plugin URI: http://hackadelic.com/solutions/wordpress/editarea
Description: WordPress integration of the EditArea real time syntax coloring code editor. Provide rich, syntax-aware code editing experience with ease.
Author: Hackadelic
Author URI: http://hackadelic.com
*/

add_action('plugins_loaded', array('HackadelicEditArea', 'start'));

class HackadelicEditArea
{
	var $info = array( // Make sure this is equal to the information in the plug-in header!
		'title' => 'Hackadelic EditArea',
		'version' => '1.1.0',
		'slug' => 'editarea');

	//-------------------------------------------------------------------------------------

	var $scripts = array(
		array('name' => 'editarea', 'path' => 'edit_area/edit_area_full.js'),
		array('name' => 'hackadelic-editarea', 'path' => 'hackadelic-editarea.js', 'deps' => array('editarea', 'jquery')),
		);

	//-------------------------------------------------------------------------------------

	var $options = array(
		'frontend' => array(
			'home'   => false,	// true or false
			'single' => false,	// true, false, or an array of post ID's or slugs
			'page'   => true,	// true, false, or an array of page ID's or slugs
			),
		'backend' => true, // true, false, or an array of admin pages
		//'backend' => array('page', 'post'), // true, false, or an array of admin pages
		);

	//-------------------------------------------------------------------------------------

	function start() {
		new HackadelicEditArea();
	}

	//-------------------------------------------------------------------------------------

	function HackadelicEditArea() {
		$this->initOptions();
		if (is_admin())
			$this->backendInit();
		else
			$this->frontendInit();
	}

	//-------------------------------------------------------------------------------------

	function initOptions() {
		$context = get_class($this);
		$options = get_option($context);
		if ($options) $this->options = $options;
	}

	//-------------------------------------------------------------------------------------

	function frontendInit() {
		$cond = $this->options['frontend'];
		//die('$cond = ' . print_r($cond, true));
		if ($cond) add_action('wp_print_scripts', array(&$this, 'enqueueScripts'));
		add_action('wp_footer', array(&$this, 'doEpilogue'));
	}

	//-------------------------------------------------------------------------------------

	function backendInit() {
		add_action('admin_menu', array(&$this, 'addAdminMenu'));

		$backend = $this->options['backend'];
		if (!$backend) return;
		$enqueueScripts = array(&$this, 'enqueueScripts');
		if (!is_array($backend))
			add_action('admin_print_scripts', $enqueueScripts);
		else foreach ($backend as $page)
			add_action("admin_print_scripts-$page.php", $enqueueScripts);
	}

	//-------------------------------------------------------------------------------------

	function frontendConditionExpr($contexts) {
		$terms = array();
		foreach ($contexts as $where => $cond):
			//printf("<PRE>$where => $cond</PRE>");
			if (!$cond) continue;
			if (is_array($cond)):
				foreach ($cond as $args):
					$args = trim($args);
					if (!$args) continue;
					$terms[] = "is_$where('$args')";
				endforeach;
			else: //if ($cond === true):
				$terms[] = "is_$where()";
			endif;
		endforeach;
		$terms = join(' || ', $terms);
		//printf("<PRE>\$condition = $terms</PRE>");
		return $terms;
	}
	function compileFrontendCondition($contexts) {
		$expr = $this->frontendConditionExpr($contexts);
		return create_function('', "return $expr;");
	}

	//-------------------------------------------------------------------------------------

	function enqueueScripts() {
		if (!is_admin()):
			$test = $this->compileFrontendCondition($this->options['frontend']);
			$cond = $test();
			if (!$cond) return;
			//wp_enqueue_script('jquery'); // should not be necessary; pulled in via deps
		endif;

		$pluginURL = WP_PLUGIN_URL . '/' . dirname(plugin_basename(__FILE__));
		foreach ($this->scripts as $script):
			$script = (object) $script;
			$path = $pluginURL.'/'.$script->path;
			//printf("<PRE>*** included script: $path</PRE>\n");
			wp_enqueue_script($script->name, $path, $script->deps);
		endforeach;
	}

	//-------------------------------------------------------------------------------------

	function doEpilogue() {
		$plugin = (object) $this->info;
		include 'common/xsig.php';
	}

	//=====================================================================================
	// ADMIN
	//=====================================================================================

	function addAdminMenu() {
		$title = $this->info['title'];
		add_options_page($title, $title, 10, __FILE__, array(&$this, 'handleOptions'));
	}

	//-------------------------------------------------------------------------------------

	function handleOptions() {
		$context = get_class($this);
		$actionURL = $_SERVER['REQUEST_URI'];
		$message = '';

		//die("Options = " . print_r($this->options, true));

		$op =& $this->options;
		if ( isset($_POST['submit']) ):
			check_admin_referer($context);
			if ( $_POST['submit'] == 'Save Settings' ):
				//printf( "<PRE>\$_POST = %s</PRE>\n", print_r($_POST, true) );
				$pause = $modified = false;

				$this->modify($op['frontend']['home'], $_POST['frontend-home'] == 'on', $modified);
				foreach (array('single', 'page') as $k):
					$value = stripslashes(trim( $_POST["frontend-$k"] ));
					$this->assumeSet($value);
					//printf("<PRE>value @ $k = %s</PRE>", print_r($value, true));
					$this->modify($op['frontend'][$k], $value, $modified);
				endforeach;
			
				$value = trim( $_POST['backend'] );
				$this->assumeSet($value);
				//die('$value = ' . print_r($value, true) . ' from "' .  $_POST['backend'] . '"');
				$this->modify($op['backend'], $value, $modified);

				//printf( "<PRE>$context::\$op = %s</PRE>\n", print_r($this->options, true) );

				if ($modified):
					$message = 'Plugin settings updated'; 
					update_option($context, $op);
					//printf( "<PRE>Saved $context::\$op = %s</PRE>\n", print_r($this->options, true) );
				endif;
			elseif ( $_POST['submit'] == 'Reset Settings' ):
				$pause = delete_option($context);
				if ($pause) $message = 'Plugin settings removed from database and reset to defaults';
			endif;
		endif;
		//printf( "<PRE>$context::\$optons = %s</PRE>\n", print_r($this->options, true) );
		
		include 'hackadelic-editarea-settings.php';
	}

	//=====================================================================================
	// Helpers
	//=====================================================================================

	function assignTo(&$var, $value) {
		//settype($value, gettype($var));
		if ($var == $value) return false;
		$var = $value;
		return true;
	}
	function modify(&$var, $value, &$modified) {
		if ($this->assignTo($var, $value)) $modified = true;
	}
	function assumeSet(&$value) {
		if ($value == '*') $value = true;
		else $value = (empty($value) || !is_string($value)) ? array() : preg_split('@\W+@', $value);
	}
}

?>